/*
============================================================================
 Name        : EGLTextureHelper.h
 Author      : Konstantin Tretyakov
 Version     :
 Copyright   : Copyright 2006, Konstantin Tretyakov
 Description : CEGLTextureHelper declaration
============================================================================
*/

#ifndef EGLTextureHelper_H
#define EGLTextureHelper_H

#include <e32std.h>
#include <e32base.h>
#include <fbs.h>
#include <aknutils.h>
#include <GLES/egl.h>
#include "data.mbg"

// Textures
#define TEXTURE_COUNT 3

// Textures must be 256x256!
#define TEX_WIDTH 256
#define TEX_HEIGHT 256

enum TTextureName {
    ETextureFloor = 0,
    ETextureWall,
    ETextureCustom
};

// Order of textures
const TMbmData TEXTURE_ORDER[] = { EMbmDataFloor, EMbmDataBricks, (TMbmData)(-1) };


// CLASS DECLARATION

class CEGLTextureHelper : public CBase
{
public: // Constructors and destructor
	~CEGLTextureHelper();
	static CEGLTextureHelper* NewL();
	static CEGLTextureHelper* NewLC();
    
    void LoadImagesL();
    void GlGenTextures();
    void GlDeleteTextures();

    void GlSetTexture(TTextureName texture);
    void GlSetNoTexture();

    // Reloads a custom-drawn texture into OpenGL
    void GlReloadCustomTexture();
private:
	CEGLTextureHelper();
	void ConstructL();

public:
    RPointerArray<CFbsBitmap> bitmaps;
    GLuint glTextures[TEXTURE_COUNT];
    
    // Device & Gc of the bitmap used for the custom texture
    CFbsBitmapDevice* iCustomTextureDevice;
    CBitmapContext* iCustomTextureGc;

    // Convenience shorthands
    inline CBitmapContext& GetCustomGc(){ return *iCustomTextureGc; };
    inline void UseTitleFont(){
        iCustomTextureGc->UseFont(CEikonEnv::Static()->TitleFont());
    }
    inline void UseNormalFont(){
        iCustomTextureGc->UseFont(CEikonEnv::Static()->NormalFont());
    }
};

#endif // EGLTextureHelper_H
