/*
============================================================================
 Name        : Labyrinth.cpp
 Author      : Konstantin Tretyakov
 Version     :
 Copyright   : Copyright 2006, Konstantin Tretyakov
 Description : The labyrinth class implementation
============================================================================
*/

#include "Labyrinth.h"
#include <e32math.h>

static const int default_maze[] = {
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    1, 1, 2, 1, 3, 0, 0, 0, 0, 1,
    1, 0, 0, 0, 2, 0, 2, 3, 0, 1,
    1, 0, 1, 0, 3, 0, 4, 0, 0, 1,
    1, 0, 3, 0, 4, 1, 4, 0, 0, 1,
    1, 0, 0, 0, 0, 0, 1, 0, 0, 1,
    1, 0, 3, 2, 2, 0, 0, 0, 0, 1,
    1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };



#define m(i, j) this->operator()(i, j)

TLabyrinth::TLabyrinth() {
    for (int i = 0; i < MAZE_WIDTH; i++)
        for (int j = 0; j < MAZE_HEIGHT; j++)
            m(i, j) = default_maze[i*MAZE_WIDTH + j];
}

void TLabyrinth::MakeRandom() {
    // Initialize random maze
    
    // Start with filled area
    for (int i = 0; i < MAZE_WIDTH ; i++)
        for (int j = 0; j < MAZE_HEIGHT; j++)
            m(i,j) = 1;
    
    // Mark boundaries
    for (int i = -1; i <= MAZE_WIDTH; i++) {
        m(i, -1) = 0;
        m(i, MAZE_HEIGHT) = 0;
    }
    
    for (int j = -1; j <= MAZE_HEIGHT; j++) {
        m(-1, j) = 0;
        m(MAZE_WIDTH, j) = 0;
    }
    
    // Do a DFS to generate random labyrinth
    m(1, 1) = 0;
    WalkMaze(1, 1);
}

void TLabyrinth::WalkMaze(int x, int y) {
    if (x < 0 || x >= MAZE_WIDTH || y < 0 || y > MAZE_HEIGHT) return;
    while (m(x-2, y) != 0 || m(x+2, y) != 0 || m(x, y-2) != 0 || m(x, y+2) != 0){
        int dx = 0, dy = 0;
        
        // Find a direction to go to
        while (! ( ((dx == 0) ^ (dy == 0)) && (m(x + 2*dx, y + 2*dy) != 0) ) ){
            dx = (Math::Random() % 3 - 1); dy = (Math::Random() % 3 - 1);
        }

        // Walk
        m(x + dx, y + dy) = 0;
        m(x + 2*dx, y + 2*dy) = 0;
        WalkMaze(x + 2*dx, y + 2*dy);
    }
}


void TLabyrinth::AddRandomSquare(int width, int height) {
    int x = Math::Random()%(MAZE_WIDTH - width - 2) + 1; // -2 because we leave a wall around
    int y = Math::Random()%(MAZE_WIDTH - height - 2) + 1;
    for (int i = 0; i < width; i++)
        for (int j = 0; j < height; j++)
            m(i + x, j + y) = 0;
}


